<?php
/**
* Web based SQLite management
* Class for manage connection to a user database
* @package SQLiteManager
* @author Frdric HENNINOT
* @version $Id: SQLiteDbConnect.class.php,v 1.6 2004/01/27 15:02:09 freddy78 Exp $ $Revision: 1.6 $
*/

/**
* Web based SQLite management
* Class for manage connection to a user database
* @package SQLiteManager
* @author Frdric HENNINOT
* @version $Id: SQLiteDbConnect.class.php,v 1.6 2004/01/27 15:02:09 freddy78 Exp $ $Revision: 1.6 $
*/
class SQLiteDbConnect {
	
	/**
	* Database name
	*
	* @access private
	* @var strig
	*/
	var $baseName;
	
	/**
	* Resource connection
	* 
	* @access public
	* @var resource
	*/
	var $connId;
	
	/**
	* Result resource
	* 
	* @access public
	* @var resource
	*/
	var $resId;
	
	/**
	* Error from connection to database
	*
	* @access public
	* @var bool
	*/
	var $connError;
	
	/**
	* User defines function list
	*
	* @access public
	* @var array
	*/
	var $functInfo;	
	
	/**
	* Class constructor
	*
	* @access public
	* @param string $base database name
	*/
	function SQLiteDbConnect($base){
		$this->baseName = $base;
		if($base == ":memory:") {
			if(DEBUG) $this->connId = sqlite_popen($base, 0666, $this->connError);
			else $this->connId = sqlite_popen($base, 0666, $this->connError);
		} else {
			if(DEBUG) $this->connId = sqlite_open($base, 0666, $this->connError);
			else $this->connId = sqlite_open($base, 0666, $this->connError);
		}
		return $this->connId;
	}
	
	/**
	* Return database properties
	*
	* @access public
	* @param string $type type of properties (table, view...)
	*/
	function getPropList($type){
		$propList = array();
		if($type!="Function"){
			$query = "SELECT name FROM (SELECT * FROM sqlite_master UNION SELECT * FROM sqlite_temp_master) WHERE type='".strtolower($type)."' ORDER BY name;";
			if($this->getResId($query)) {
				while($ligne = SQLiteFetchFunction($this->resId, SQLITE_ASSOC))  {
					$propList[] = $ligne["name"];	
				}
			}
		} else {
			$query = "SELECT funct_name FROM user_function WHERE (base_id=".$GLOBALS["dbsel"]." OR base_id IS NULL)";	
			if( ($res = SQLiteExecFunction($GLOBALS["db"], $query)) && (SQLiteNumFunction($res)>0)){
				while($ligne = SQLiteFetchFunction($res, SQLITE_ASSOC)) $propList[] = $ligne["funct_name"];
			}				
		}
		return $propList;
	}
	
	/**
	* Close connection to database
	*
	* @access public
	*/
	function close(){
		if(DEBUG) sqlite_close($this->connId);
		else @sqlite_close($this->connId);
		return;
	}
	
	/**
	* Exec a query and return the resource result
	*
	* @access public
	* @param string $query 
	* @param bool $buffer
	*/
	function getResId($query, $buffer=true){
		if($buffer) {
			if(DEBUG) $this->resId = sqlite_query($this->connId, $query);
			else $this->resId = @sqlite_query($this->connId, $query);
		} else {
			if(DEBUG) $this->resId = sqlite_unbuffered_query($this->connId, $query);
			else $this->resId = @sqlite_unbuffered_query($this->connId, $query);
		}
		return $this->resId;		
	}
	
	/**
	* return array of result
	*
	* @access public
	* @param string $type
	*/
	function getArray($type = ''){
		if(is_resource($this->resId)){
			$tabOut = array();
			while($row = SQLiteFetchFunction($this->resId, (($type=='')? SQLITE_ASSOC : $type ))) $tabOut[] = $row;
			return $tabOut;
		} else {
			return false;
		}
	}
	
	/**
	* Manage ALTER TABLE, not exist in SQLite
	* 
	* @access public
	* @param string $table Table name
	* @param string $newDefinition SQL definition of CREATE TABLE
	* @param array $oldColumn Array of all the column
	* @param array $newColumn Array of all new column
	*/
	function alterTable($table, $newDefinition, $oldColumn, $newColumn){
		$query[] = "BEGIN TRANSACTION;\n";
		$query[] = "CREATE TEMPORARY TABLE backup AS SELECT * FROM ".$table.";\n";
		$query[] = "DROP TABLE ".$table.";\n";	
		$query[] = $newDefinition."\n";		
		$query[] = "INSERT INTO ".$table." (".implode(", ", $newColumn).") SELECT ".implode(", ", $oldColumn)." FROM backup;\n";
		$query[] = "DROP TABLE backup;\n";
		$query[] = "COMMIT TRANSACTION;\n";
		$noerror = true;
		foreach($query as $req) if(!SQLiteExecFunction($this->connId, $req)) $noerror = false;
		return $noerror;
	}
	
	/**
	* Return properties of all user defined function
	*
	* @access public
	*/
	function getUDF(){
		$query = "SELECT id, base_id, funct_type, funct_name, funct_code, funct_final_code, funct_num_args FROM user_function WHERE (base_id=".$GLOBALS["dbsel"]." OR base_id IS NULL)";
			if( ($res = sqlite_query($GLOBALS["db"], $query)) && (SQLiteNumFunction($res)>0)){
				while($ligne = SQLiteFetchFunction($res, SQLITE_ASSOC)) $this->functInfo[$ligne["id"]] = $ligne;
			}
		return $this->functInfo;			
	}
	
	/**
	* Insert user defines function in the executable context to use this in SQL
	*
	* @access public
	*/
	function includeUDF(){
		$tabFunct = $this->getUDF();
		if(is_array($tabFunct)){
			foreach($tabFunct as $infoFunct){
				switch($infoFunct["funct_type"]){
					case 1:	// function utilisateur standard
						$code = $infoFunct["funct_code"];
						$funcName = $this->_getPHPfunctionName($code);
						eval($code);
						if(DEBUG) sqlite_create_function($this->connId, $infoFunct["funct_name"], $funcName, $infoFunct["funct_num_args"]);
						else @sqlite_create_function($this->connId, $infoFunct["funct_name"], $funcName, $infoFunct["funct_num_args"]);
						break;
					case 2:	// function utilisateur d'aggregation
						$codeStep = $infoFunct["funct_code"];
						$codeFinal = $infoFunct["funct_final_code"];
						eval($codeStep);
						eval($codeFinal);
						if(DEBUG) sqlite_create_aggregate ( $this->connId, $infoFunct["funct_name"], $this->_getPHPfunctionName($codeStep), $this->_getPHPfunctionName($codeFinal), $infoFunct["funct_num_args"]);
						else @sqlite_create_aggregate ( $this->connId, $infoFunct["funct_name"], $this->_getPHPfunctionName($codeStep), $this->_getPHPfunctionName($codeFinal), $infoFunct["funct_num_args"]);
						break;
				}
			}
		}
	}
	
	/**
	* Retreive function name from PHP code
	*/
	function _getPHPfunctionName($code){
		$codeSearch = str_replace("FUNCTION", "function", $code);
		preg_match("/function[[:space:]](.*)\((.*)\)[[:space:]]*{/", $codeSearch, $value);
		return $value[1];		
	}
	
	/**
	* Escape string To SQLite
	* @access public
	* @param string $string
	*/
	function formatString($string){
		return @sqlite_escape_string(stripslashes($string));
	}
}
?>
