<?php
/**
* Web based SQLite management
* @package SQLiteManager
* @author Frdric HENNINOT
* @version $Id: sql.class.php,v 1.16 2004/03/09 18:40:11 freddy78 Exp $ $Revision: 1.16 $
*/

/**
* Abstract layer for manage user's SQLite database
* @package SQLiteManager
* @author Frdric HENNINOT
* @version $Id: sql.class.php,v 1.16 2004/03/09 18:40:11 freddy78 Exp $ $Revision: 1.16 $
*/
class sql {

	/**
    * @access public
	* @var objet
	* Class instance of database connection
	*/
	var $connId;
	
	/**
    * @access private
	* @var string
	* current query
	*/
	var $query;
	
	/**
    * @access private
	* @var bool
	* if true, all query are journalised into file
	*/
	var $journalised;
	
	/**
    * @access private
	* @var string
	* filename where query are store when journalised
	*/
	var $journalFile;
	
	/**
    * @access public
	* @var bool
	* true if query return data
	*/
	var $withReturn;
	
	/**
    * @access public
	* @var bool
	* true if the current query content are multiple
	*/
	var $multipleQuery;
	
	/**
    * @access private
	* @var integer
	* Number lines has been changes by the current query
	*/
	var $changesLine;
	
	/**
    * @access private
	* @var integer
	* Number of query into the curent multiple query
	*/
	var $nbQuery;
	
	/**
    * @access private
	* @var bool
	* true if has an error in the current query
	*/
	var $error;
	
	/**
    * @access private
	* @var integer
	* Error line number for multiple query
	*/
	var $lineError;
	
	/**
    * @access private
	* @var string
	* SQLite error message
	*/
	var $errorMessage;
	
	/**
	* Class constructor	
	*
    * @access public
	* @param object $handle instance of connection class
	* @param string $query 
	* @param string $journal filename of the journalise file
	*/
	function sql(&$handle, $query, $journal=""){
		$this->connId = $handle;
		$this->query = trim($query);
		if(eregi("^(select|explain|pragma)[[:space:]]", $this->query)) $this->withReturn = true;
		else $this->withReturn = false;
		if($journal != ""){
			$this->journalised = true;
			$this->journalFile = $journal;
		} else $this->journalised = false;
		return;
	}
	
	/**
	* Verify and exec query	
	*
    * @access public
	*/
	function verify(){
		if(is_resource($this->connId->connId)){
			if($this->query != ""){
				$parsing =& new ParsingQuery($this->query, ((isset($_POST["sqltype"]))? $_POST["sqltype"] : 1 ));
				$tabQuery = $parsing -> convertQuery();
				if(!is_array($tabQuery)){
					$this->multipleQuery = false;
					$this->query = $tabQuery;
					return $this->execQuery($tabQuery);
				} else {
					$this->multipleQuery = true;
					$this->execQuery("BEGIN TRANSACTION;");
					$error = false;
					$lineNum = 1;
					$this->changesLine = $queryNum = 0;
					foreach($tabQuery as $query){
						if(!empty($query) && substr(trim($query), 0, 1)!="#"){
							if($this->_checkBeginQuery($query)){							
								$queryNum++;
								if(isset($commitafter) && $commitafter){
									$this->execQuery("COMMIT TRANSACTION;");
									$this->execQuery("BEGIN TRANSACTION;");
									$commitafter=false;
								}
							}							
							if($this->_checkBeginQuery($query, "CREATE|DROP")) $commitafter = true;
													
							if($this->execQuery($query)){ 
								$error = true;
								$this->lineError[] = $lineNum;
							}
							if(DEBUG) $this->changesLine += sqlite_changes($this->connId->connId);
							else $this->changesLine += @sqlite_changes($this->connId->connId);
							$lineNum++;
						}
					}
					if($error) $this->execQuery("ROLLBACK TRANSACTION;");
					else $this->execQuery("COMMIT TRANSACTION;");
					$this->error = $error;
					$this->withReturn = false;
					$this->nbQuery = $queryNum;
				}					
			} else {
				$this->error = true;
				$this->errorMessage = $GLOBALS["traduct"]->get(64);
			}
		} else {
			$this->error = true;
			$this->errorMessage = $GLOBALS["traduct"]->get(65);
		}			
		return $this->error;
	}
	
	/**
	* Exec, manage error and journalised	
	*
    * @access public
	* @param string $string query
	*/
	function execQuery($query){
		$queryExec = $this->cleanup($query);
		$this->queryLog($queryExec);
		if($this->connId->getResId($queryExec)){
			$this->error = false;
		} else {
			$this->error = true;
			$errorCode = @sqlite_last_error($this->connId->connId);
			$this->errorMessage = $GLOBALS["traduct"]->get(9)." ".$errorCode." : ".@sqlite_error_string($errorCode);			
		}
		return $this->error;
	}
	

	/**
	* Cleanup POST query
	* and convert MySQL type into SQLite type
	*
    * @access public
	* @param array $data data table
	* @param string $width width of the end table (px or %)
	*/
	function cleanup($query){
		$query = stripslashes($query);
		if(!isset($_POST["sqltype"])) $_POST["sqltype"]=1;
		if($_POST["sqltype"]==2){
			$query = str_replace("\'", "''", $query);
			$query = str_replace("\\\"", "\"\"", $query);
		}
		return $query;
	}
	/**
	* Manual query Form	
	*
    * @access public
	* @param string $query
	*/
	function getForm($query){
		echo "<div align='center'><table><tr><td>";
		echo "	<form name=\"sql\" action=\"main.php?dbsel=".$GLOBALS["dbsel"]."&table=".$GLOBALS["table"]."\" method=\"POST\" ENCTYPE=\"multipart/form-data\">
				".$GLOBALS["traduct"]->get(66)."</b><br>
				<textarea name=\"DisplayQuery\" cols=80 rows=7 onfocus=\"document.sql.DisplayQuery.select();\">".$this->cleanup($query)."</textarea><br>
				".$GLOBALS["traduct"]->get(67)." :
				<input type=\"file\" name=\"sqlFile\"><br>
				".$GLOBALS["traduct"]->get(68)."<input type='radio' name='sqltype' value=1 ".(((!isset($_POST["sqltype"])) || ($_POST["sqltype"]==1))? "checked" : "" )."> - MySQL<input type='radio' name='sqltype' value='2'".(((isset($_POST["sqltype"])) && ($_POST["sqltype"]==2))? " checked" : "" )."><br>
				<input type=\"hidden\" name=\"action\" value=\"sql\">
				<input type=\"submit\" value=\"".$GLOBALS["traduct"]->get(69)."\">				
				</form>";
		echo "</td></tr></table></div>";
	}
	
	/**
	* Verify if the result can be modify or deleted
	* if true, return the table name else return false	
	*
    * @access public
	* @param string $query
	*/
	function checkAccessResult($query){	
		if(eregi("JOIN|GROUP", $query)) return false;
		$match = "WHERE|ORDER|LIMIT";
		if(eregi($match, $query)) preg_match("/FROM(.*)(".$match.")/i", $query, $result);
		else preg_match("/FROM(.*)/i", $query, $result);
		$listTable = $result[1];
		if(strpos($listTable, ",")) return false;
		$tableNAme = trim($listTable);
		if($res = $this->connId->getResId("SELECT type FROM sqlite_master WHERE name LIKE '".$tableNAme."'"));
		if(@sqlite_fetch_string($res) != "table") return false; 
		else return $tableNAme;		
	}
	
	/**
	* Log string into journal file	
	*
    * @access public
	* @param string $string
	*/
	function queryLog($string){
		if($this->journalised){
			$fp = fopen($this->journalFile, "a+");
			fwrite($fp, $string."\n");
			fclose($fp);
		}
	}
	
	/**
	* Verify if the string param is a start of string
	* and if param motif is set, check if the start query content this	
	*
    * @access public
	* @param string $req query
	* @param string $motif
	*/
	function _checkBeginQuery($req, $motif=""){
		if(eregi("select|insert|update|delete|create|drop|replace|pragma", $req)){
			if($motif != "") return eregi($motif, $req);
			else return true;			
		} else return false;
	}
	
	/**
	* Display result when the query is multiple	
	*
    * @access public
	*/
	function DisplayMultipleResult(){
		echo "
			<table width=60% align=\"center\">
				<tr><td bgcolor=\"lightblue\" >".$this->nbQuery." ".$GLOBALS["traduct"]->get(70)."</td></tr>
				<tr><td bgcolor=\"CCCCCC\"><span class=\"sqlsyntaxe\">&nbsp;".$this->changesLine." ".$GLOBALS["traduct"]->get(71)."</span></td></tr>
			</table>";
		return;
	}
}
?>
